//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
/**
@file         LxS_Param.cpp
@brief        Implementation of a Class that represents a LxS-Parameter
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//! Main Namespace for LxS-Lib
namespace LxS
{
  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       UINT16 * Param::internal_CreateCopy(UINT16 * wds, int len)
  @brief    Create Copy
  @param    wds - original pointer
  @param    len - length
  @return   New UINT16-Array
  @details
  Create a Copy from given Array.
  */
  UINT16 * Param::internal_CreateCopy(UINT16 * wds, int len)
  {
    UINT16 * ret = NULL;
    if ((len > 0) && (wds != NULL))
    {
      ret = new UINT16[len]; // unmanaged : create buffer
      memcpy(ret, wds, (len << 1)); // copy original data to buffer
      //delete[] ret; // unmanaged : free buffer
    };
    //! @remark : user should free ressources with : delete[] ret; after using
    return (ret); //! @remark : !!! UNMANAGED RETURN !!!
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       Param::Param(UINT16 * structure, UINT16 * data)
  @brief    Constructor for LxS::Param-Class
  @param    structure - pointer to structure of parameter
  @param    data - pointer to data of parameter
  @details
  Create new Instance of LxS::Param-Class from given Structure and Data.
  */
  Param::Param(UINT16 * structure, UINT16 * data)
  {
    this->data = NULL;
    this->next = NULL;
    if (structure != NULL)
    {
      this->id = structure[0];
      this->dt = structure[1];
      this->cnt = structure[2];
      this->min = ((((UINT32)(structure[3])) << 16) + ((UINT32)(structure[4])));
      this->max = ((((UINT32)(structure[5])) << 16) + ((UINT32)(structure[6])));
      this->flds = structure[7];
      this->data = this->internal_CreateCopy(data, this->cnt); // unmanaged : copy data
      //! @remark : user should free ressources with : delete[] this->data; after using
    };
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       Param::~Param()
  @brief    Destructor for LxS::Param-Class
  @details
  Method killing a Instance of LxS::Param-Class.
  */
  Param::~Param()
  {
    if (this->next != NULL) // free all members from set
    {
      this->next->~Param(); // destructor ...
    };
    this->next = NULL;
    delete[] this->data; // free own data-copy
  }
}
