//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
//--------------------------------------------------------------------------------------------------------------------
/**
@file         LxS_Param.h
@brief        Definition of a Class that represents a LxS-Parameter
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#ifndef   __DEVEL_LXS_PARAM_H__
#define   __DEVEL_LXS_PARAM_H__

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//---------------------------------------------------------------------------------------------------------------------
#ifdef    __cplusplus
extern "C"
{
#endif // __cplusplus

  //! Main Namespace for LxS-Lib
  namespace LxS
  {
    //-----------------------------------------------------------------------------------------------------------------
    /**
    @class      Param
    @brief      Class that represents a LxS-Parameter
    @details    A parameter has 2 section. First the definition of a parameter (with elements like minimal limit,
                maximal limit and so on). And second the value(s) of the parameter.
    */
    class DLLSPEC   Param
    {
    private:
      UINT16 *      internal_CreateCopy(UINT16 * wds, int len);

    public:
      Param *       next;     //!< next parameter in a set
      UINT16 *      data;     //!< data-pointer to values
      UINT16        id;       //!< parameter id
      UINT16        dt;       //!< parameter datatype
      UINT16        cnt;      //!< count of data
      UINT32        min;      //!< minimal allowed value
      UINT32        max;      //!< maximal allowed value
      UINT16        flds;     //!< count of fields

      Param(UINT16 * structure, UINT16 * data);
      ~Param();
    };
  }

#ifdef    __cplusplus
}
#endif // __cplusplus

#endif // __DEVEL_LXS_PARAM_H__