//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
//--------------------------------------------------------------------------------------------------------------------
/**
@file         LxS_ParamSet.h
@brief        Definition of a Class that represents a LxS-Parameter-Set
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#ifndef   __DEVEL_LXS_PARAM_SET_H__
#define   __DEVEL_LXS_PARAM_SET_H__

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//---------------------------------------------------------------------------------------------------------------------
#ifdef    __cplusplus
extern "C"
{
#endif // __cplusplus

  //! Main Namespace for LxS-Lib
  namespace LxS
  {
    //-----------------------------------------------------------------------------------------------------------------
    /**
    @class      ParamSet
    @brief      Class that represents a LxS-Parameter-Set
    @details    A parameter set consists of an array of parameters (LxS::Param)
    */
    class DLLSPEC   ParamSet
    {
    private:
      BYTE *        prm_struct;     //!< own copy of structure
      int           prm_length;     //!< length of structure
      LxS::Param *  prm_list;       //!< parameter-set
      BYTE *        internal_CreateCopy(BYTE * bts, int len);

    public:
      int           id;             //!< own id
      int           stype;          //!< own structure type
      ParamSet(int id, int stype, BYTE * field, int f_len, BYTE * data, int d_len);
      ~ParamSet();
      Param *       GetSettings(void);
      bool          CheckStructure(ParamSet * prm);
      BYTE *        GenerateParameterPackage(int * len);
    };
  }

#ifdef    __cplusplus
}
#endif // __cplusplus

#endif // __DEVEL_LXS_PARAM_SET_H__
