//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
/**
@file         LxS_Stuff.cpp
@brief        GUI-Example for using LxS-Lib.
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#include "stdafx.h"

LxS::Device * lxs = NULL;
HWND last_hwnd;
BYTE * before_z = NULL;
BYTE * before_x = NULL;
RECT * update_rect = NULL;
bool z_mode = false;
UINT16 last_scan = 0;

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       void SetWorkingMode(HWND hWnd, bool state)
@brief    Sync GUI-Mode to Sensor-Mode
@param    hWnd - Window-Handler
@param    state - New State
@details
Synchronize GUI-Komponents with actual Sensor-Mode.
*/
void SetWorkingMode(HWND hWnd, bool state)
{
  EnableWindow(GetDlgItem(hWnd, IDC_MY_EDT_014), !state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_EDT_015), !state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_EDT_016), !state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_001), !state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_002), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_003), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_004), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_005), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_006), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_007), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_008), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_EDT_009), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_010), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_SCR_011), state);
  EnableWindow(GetDlgItem(hWnd, IDC_MY_CHK_012), state);

}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       void CreateTestGUI(HWND hWnd)
@brief    Create GUI
@param    hWnd - Window-Handler
@details
Create new GUI, place all Buttons/Components and register all Events.
*/
void CreateTestGUI(HWND hWnd)
{
  HWND x;
  x = CreateWindowEx(NULL, L"STATIC", L"info ...",
    (WS_VISIBLE | WS_CHILD | SS_LEFT | SS_SUNKEN), 10, 10, 310, 24,
    hWnd, (HMENU)IDC_MY_EDT_000, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"STATIC", L"LxS-IP:",
    (WS_VISIBLE | WS_CHILD | SS_LEFT | SS_SUNKEN), 10, 54, 150, 24,
    hWnd, (HMENU)NULL, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(WS_EX_CLIENTEDGE, L"EDIT", L"192.168.60.3",
    (WS_CHILD | WS_VISIBLE), 170, 50, 150, 32,
    hWnd, (HMENU)IDC_MY_EDT_014, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"STATIC", L"LxS-Port:",
    (WS_VISIBLE | WS_CHILD | SS_LEFT | SS_SUNKEN), 10, 94, 80, 24,
    hWnd, (HMENU)NULL, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(WS_EX_CLIENTEDGE, L"EDIT", L"9008",
    (WS_CHILD | WS_VISIBLE), 100, 90, 60, 32,
    hWnd, (HMENU)IDC_MY_EDT_015, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"STATIC", L"PC-Port:",
    (WS_VISIBLE | WS_CHILD | SS_LEFT | SS_SUNKEN), 170, 94, 80, 24,
    hWnd, (HMENU)NULL, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(WS_EX_CLIENTEDGE, L"EDIT", L"5634",
    (WS_CHILD | WS_VISIBLE), 260, 90, 60, 32,
    hWnd, (HMENU)IDC_MY_EDT_016, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Connect",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 10, 130, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_001, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Disconnect",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 170, 130, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_002, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Mode: COMMAND",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 10, 170, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_003, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Mode: MEASURE",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 170, 170, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_004, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Laser: ON",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 10, 210, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_005, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Laser: OFF",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 170, 210, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_006, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"STATIC", L"Task:",
    (WS_VISIBLE | WS_CHILD | SS_LEFT | SS_SUNKEN), 10, 254, 70, 24,
    hWnd, (HMENU)NULL, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(WS_EX_CLIENTEDGE, L"EDIT", L"0",
    (WS_CHILD | WS_VISIBLE), 90, 250, 70, 32,
    hWnd, (HMENU)IDC_MY_EDT_009, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Trigger",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 170, 250, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_010, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Get Task",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 10, 290, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_007, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"BUTTON", L"Set Task",
    (WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON), 170, 290, 150, 32,
    hWnd, (HMENU)IDC_MY_BTN_008, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"SCROLLBAR", L"",
    (WS_CHILD | WS_VISIBLE | SBS_HORZ | SBS_BOTTOMALIGN), 10, 320, 310, 32,
    hWnd, (HMENU)IDC_MY_SCR_011, GetModuleHandle(NULL), NULL);
  SetScrollRange(x, SB_CTL, MY_SCR_MIN, MY_SCR_MAX, TRUE);
  SetScrollPos(x, SB_CTL, MY_SCR_MIN, TRUE);

  x = CreateWindowEx(NULL, L"BUTTON", L"Z / XZ",
    (WS_VISIBLE | WS_CHILD | BS_CHECKBOX), 10, 360, 150, 32,
    hWnd, (HMENU)IDC_MY_CHK_012, GetModuleHandle(NULL), NULL);

  x = CreateWindowEx(NULL, L"STATIC", L"Scan: -",
    (WS_VISIBLE | WS_CHILD | SS_CENTER | SS_SUNKEN), 170, 360, 150, 32,
    hWnd, (HMENU)IDC_MY_EDT_013, GetModuleHandle(NULL), NULL);

  // Rect for Live-Picture
  update_rect = new RECT();
  update_rect->left = 320; update_rect->right = 740;
  update_rect->top = 0; update_rect->bottom = 450;

  SetWorkingMode(hWnd, FALSE);
  InvalidateRect(hWnd, new RECT(), true);
  UpdateWindow(hWnd);
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       bool ExecuteAction(HWND hWnd, int id, int evnt)
@brief    Execute User-Action
@param    hWnd - Window-Handler
@param    id - Component-ID
@param    evnt - Event/Argument
@return   Execution State
@details
Function will execute User-Action true LxS-device and send response back to GUI.
*/
bool ExecuteAction(HWND hWnd, int id, int evnt)
{
  bool ret = true;
  switch (id)
  {
  case IDC_MY_BTN_001:
    {
      EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_001), FALSE);
      bool result = false;

      // Get infos from GUI ...
      wchar_t buffer[256]; char lx_addr[20]; int lx_port = 0; int pc_port = 0;
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_014), WM_GETTEXT, 256, (LPARAM)buffer);
      for(int i = 0; i<20; i++) { lx_addr[i] = ((char)(buffer[i])); };
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_015), WM_GETTEXT, 256, (LPARAM)buffer);
      int p = 0; while(buffer[p] != 0) { lx_port *= 10; lx_port += ((buffer[p] & 0x00FF) - 0x0030); p++; };
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_016), WM_GETTEXT, 256, (LPARAM)buffer);
      p = 0; while(buffer[p] != 0) { pc_port *= 10; pc_port += ((buffer[p] & 0x00FF) - 0x0030); p++; };

      // Clean-Up !
      if (lxs != NULL) { lxs->~Device(); };

      // Create Device
      lxs = new LxS::Device(pc_port);
      lxs->SetDataHandler(NULL);

      // Create Socket
      if (lxs->Create(lx_addr, lx_port))
      {
        if (lxs->Connect()) 
        {
          lxs->SetDataHandler(MyCB);
          result = true;
        };
      };
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)(result ? L"connected ..." : L"not connected ..."));
      SetWorkingMode(hWnd, result);
      break;
    }
  case IDC_MY_BTN_002:
    {
      lxs->Disconnect();
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"disconnected ...");
      SetWorkingMode(hWnd, FALSE);
      break;
    }
  case IDC_MY_BTN_003:
    {
      lxs->SetSensorMode(LxS::LXS_MODE_COMMAND);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set command mode ...");
      break;
    }
  case IDC_MY_BTN_004:
    {
      lxs->SetSensorMode(LxS::LXS_MODE_MEASURE);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set measure mode ...");
      break;
    }
  case IDC_MY_BTN_005:
    {
      lxs->SetLaserGate(true);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set laser ON ...");
      break;
    }
  case IDC_MY_BTN_006:
    {
      lxs->SetLaserGate(false);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set laser OFF ...");
      break;
    }
  case IDC_MY_BTN_007:
    {
      wchar_t buffer[256];
      UINT16 tnr = lxs->GetInspectionTask();
      swprintf(buffer, 256, L"%d", tnr);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_009), WM_SETTEXT, NULL, (LPARAM)buffer);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"get task ...");
      break;
    }
  case IDC_MY_BTN_008:
    {
      wchar_t buffer[256];
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_009), WM_GETTEXT, 256, (LPARAM)buffer);
      int tnr = 0;
      int p = 0;
      while(buffer[p] != 0) { tnr *= 10; tnr += ((buffer[p] & 0x00FF) - 0x0030); p++; };
      lxs->SetInspectionTask((UINT16)tnr);
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set task ...");
      break;
    }
  case IDC_MY_BTN_010:
    {
      EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_010), FALSE);
      bool x = lxs->EthernetTrigger();
      SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"trigger ...");
      EnableWindow(GetDlgItem(hWnd, IDC_MY_BTN_010), TRUE);
      break;
    }
  case IDC_MY_SCR_011:
    {
      HWND sc = GetDlgItem(hWnd, IDC_MY_SCR_011);
      int cursor = GetScrollPos(sc, SB_CTL);
      switch (LOWORD(evnt))
      {
      case SB_TOP:
        {
          cursor = MY_SCR_MIN;
          break;
        }
      case SB_BOTTOM:
        {
          cursor = MY_SCR_MAX;
          break;
        }
      case SB_LINEUP:
        {
          if (cursor > MY_SCR_MIN) { cursor--; };
          break;
        }
      case SB_LINEDOWN:
        {
          if (cursor < MY_SCR_MAX) { cursor++; };
          break;
        }
      case SB_PAGEUP:
        {
          if (cursor > (MY_SCR_MIN + 5)) { cursor -= 5; } else { cursor = MY_SCR_MIN; };
          break;
        }
      case SB_PAGEDOWN:
        {
          if (cursor < (MY_SCR_MAX - 5)) { cursor += 5; } else { cursor = MY_SCR_MAX; };
          break;
        }
      case SB_THUMBPOSITION:
      case SB_THUMBTRACK:
        {
          cursor = HIWORD(evnt);
          break;
        }
      case SB_ENDSCROLL:
        {
          break;
        }
      }
      SetScrollPos(sc, SB_CTL, cursor, TRUE);
      int val = ((cursor == 2) ? (739) : (((cursor - 3) * 328) + 973));
      LxS::Param * prm = lxs->GetSingleTaskParam(3006);
      if (prm != NULL)
      {
        val = ((val > (int)prm->max) ? ((int)prm->max) : (val));
        val = ((val < (int)prm->min) ? ((int)prm->min) : (val));
        prm->data[0] = ((UINT16)(val));
        if (lxs->SetSingleTaskParam(3006, prm, false))
        {
          SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set exposure time (OK) ...");
        }
        else
        {
          SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"set exposure time (NO) ...");
        };
        prm->~Param();
      };
      break;
    }
  case IDC_MY_CHK_012:
    {
      UINT chk = IsDlgButtonChecked(hWnd, IDC_MY_CHK_012);
      if (chk)
      {
        z_mode = false;
        CheckDlgButton(hWnd, IDC_MY_CHK_012, BST_UNCHECKED);
        SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"show ZX ...");
      }
      else
      {
        z_mode = true;
        CheckDlgButton(hWnd, IDC_MY_CHK_012, BST_CHECKED);
        SendMessage(GetDlgItem(hWnd, IDC_MY_EDT_000), WM_SETTEXT, NULL, (LPARAM)L"show only Z ...");
      };
      InvalidateRect(hWnd, NULL, true);
      break;
    }
  default:
    {
      ret = false;
      break;
    }
  };
  return (ret);
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
@brief    Data-Handler
@param    dt - Type of Package
@param    ptr - Pointer to Data-Field
@param    len - Length of Data-Filed
@details
Data-Handler will be called from Lib after receiving new Measure-Data from Sensor.
*/
void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
{
  if (dt == LxS::LXS_DATA_Z)
  {
    delete[] before_z;
    before_z = ptr;
  };
  if (dt == LxS::LXS_DATA_X)
  {
    UINT16 sn = lxs->GetScanNumber();
    if (sn != last_scan)
    {
      wchar_t buffer[256];
      last_scan = sn;
      swprintf(buffer, 256, L"Scan: %d", last_scan);
      SendMessage(GetDlgItem(last_hwnd, IDC_MY_EDT_013), WM_SETTEXT, NULL, (LPARAM)buffer);
    };
    delete[] before_x;
    before_x = ptr;
    InvalidateRect(last_hwnd, update_rect, false);
  };

  if (dt == LxS::LXS_DATA_ZX)
  {
    UINT16 sn = lxs->GetScanNumber();
    if (sn != last_scan)
    {
      wchar_t buffer[256];
      last_scan = sn;
      swprintf(buffer, 256, L"Scan: %d", last_scan);
      SendMessage(GetDlgItem(last_hwnd, IDC_MY_EDT_013), WM_SETTEXT, NULL, (LPARAM)buffer);
    };
    delete[] before_z;
    // no delete, because before_x points on the same buffer.
    before_z = ptr;
    before_x = ptr + 480;
    InvalidateRect(last_hwnd, update_rect, false);
  };
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       void OnPaint(HWND hWnd)
@brief    GUI-Revalidation
@param    hWnd - Window-Handle
@details
GUI will be repainted after Data/ or User-Changes
*/
void OnPaint(HWND hWnd)
{
  PAINTSTRUCT ps;
  last_hwnd = hWnd;
  HDC hdc = BeginPaint(hWnd, &ps);
  Rectangle(hdc, 8, 329, 322, 354);

  HDC hdcMem = CreateCompatibleDC(hdc);
  HBITMAP hbmMem = CreateCompatibleBitmap(hdc, 405, 435);
  HANDLE hOld = SelectObject(hdcMem, hbmMem);
  HPEN pn = CreatePen(PS_SOLID, 1, RGB(128, 128, 128));
  //HPEN px = CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
  Rectangle(hdcMem, 0, 0, 400, 430);
  SelectObject(hdcMem, pn);
  if (z_mode)
  {
    for (int i = 30; i < 420; i += 20)
    {
      MoveToEx(hdcMem, 10, i, NULL); LineTo(hdcMem, 390, i);
      MoveToEx(hdcMem, i, 10, NULL); LineTo(hdcMem, i, 420);
    }

    MoveToEx(hdcMem, 10, 10, NULL);
    LineTo(hdcMem, 390, 10);
    LineTo(hdcMem, 390, 420);
    LineTo(hdcMem, 10, 420);
    LineTo(hdcMem, 10, 10);

    if (before_z != NULL)
    {
      int new_x = 386;  // Die X Position
      UINT16 * ptr_z = ((UINT16 *)(before_z));
      for (int i = 0; i < 376; i++)
      {
        UINT16 pz = (ptr_z[i] / 20);
        SetPixel(hdcMem, new_x, pz, RGB(255,0,0));
        SetPixel(hdcMem, new_x--, pz+1, RGB(255,0,0));
      };
    };

    Rectangle(hdcMem, 229, 29, 372, 52);
  }
  else
  {
    MoveToEx(hdcMem, 160, 100, NULL);
    LineTo(hdcMem, 240, 100);
    LineTo(hdcMem, 355, 420);
    LineTo(hdcMem, 45, 420);
    LineTo(hdcMem, 160, 100);	

    MoveToEx(hdcMem, 200, 100, NULL); LineTo(hdcMem, 200, 420);
    MoveToEx(hdcMem, 180, 100, NULL); LineTo(hdcMem, 123, 420);
    MoveToEx(hdcMem, 220, 100, NULL); LineTo(hdcMem, 277, 420);

    MoveToEx(hdcMem, 143, 150, NULL); LineTo(hdcMem, 256, 150);
    MoveToEx(hdcMem, 125, 200, NULL); LineTo(hdcMem, 274, 200);
    MoveToEx(hdcMem, 107, 250, NULL); LineTo(hdcMem, 293, 250);
    MoveToEx(hdcMem, 88, 300, NULL); LineTo(hdcMem, 311, 300);
    MoveToEx(hdcMem, 70, 350, NULL); LineTo(hdcMem, 330, 350);
    MoveToEx(hdcMem, 52, 400, NULL); LineTo(hdcMem, 348, 400);

    if ((before_x != NULL) && (before_z != NULL))
    {
      INT16 * ptr_x = ((INT16 *)(before_x));
      UINT16 * ptr_z = ((UINT16 *)(before_z));
      for(int i = 0; i< 376; i++)
      {
        INT16 px = (ptr_x[i] / -20) + 200;
        UINT16 pz = (ptr_z[i] / 20 );
        SetPixel(hdcMem, px, pz, RGB(255,0,0));
        SetPixel(hdcMem, px, pz+1, RGB(255,0,0));
      };
    };
  };

  SetTextColor(hdcMem, RGB(0,0,0));
  TextOut(hdcMem, 235, 31, L"Leuze", 5);
  SetTextColor(hdcMem, RGB(227,0,64));
  TextOut(hdcMem, 287, 31, L"electronic", 10);

  BitBlt(hdc, 330, 10, 401, 431, hdcMem, 0, 0, SRCCOPY);
  DeleteObject(pn);
  //DeleteObject(px);
  SelectObject(hdcMem, hOld);
  DeleteObject(hbmMem);
  DeleteDC(hdcMem);
  EndPaint(hWnd, &ps);
}